// bufchan.go - send, receive with buffered channel
package main
import ("fmt"; "sync"; "time")

var wg sync.WaitGroup

func doSomething() {
    time.Sleep(2 * time.Second)
}

func main() {
    wg.Add(2)                       // add goroutines

    ch := make(chan string, 100)    // buffered channel
    go sendMessages(ch)
    doSomething()
    go getMessages(ch)

    wg.Wait()                       // wait for goroutines to finish
}

func sendMessages(ch chan string) {
    defer wg.Done()    
    messages := []string { "Message 1", "Message 2", "Message 3" }
    for _, msg := range messages {
        fmt.Println("sending message...")
        ch <- msg
    }
    close(ch)                       // end of messages
}

func getMessages(ch chan string) {
    defer wg.Done()
    for msg := range ch {
        fmt.Println(msg)
    }
}

/*************************************

    $ go run bufchan.go
    sending message...
    sending message...
    sending message...
    Message 1
    Message 2
    Message 3

*/
