// select.go - select statement
package main
import ("fmt"; "strconv"; "math/rand"; "time")

func init() {
    rand.Seed(time.Now().UTC().UnixNano())
}

func send(chs chan<- string) {
    r := rand.Intn(300)
    time.Sleep(time.Duration(r) * time.Millisecond)
    chs <- "sent message after " + strconv.Itoa(r) + "ms"
}

func main() {
    timeout := time.NewTimer(200 * time.Millisecond).C
    ch := make(chan string)
    go send(ch)

    select {
        case msg := <-ch:
            fmt.Println(msg)
        case <-timeout:
            fmt.Println("timeout after 200ms")
    }
}

/*************************************

    $ go run select.go
    sent message after 122ms
    sent message after 173ms
    sent message after 47ms
    timeout after 200ms
    sent message after 100ms
    sent message after 59ms
    timeout after 200ms
    sent message after 124ms

*/
